/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.utils;

import com.rempl.commons.ProxyConfig;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.lang.StringUtils;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcCommonsTransportFactory;

/**
 * A HTTP transport factory based on Jackarta commons which is proxy aware.
 * Setting standard http.proxyHost and http.proxyPort will be recognized
 * and HttpClient will have the appropriate proxy data set.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: URLBuilder.java 1357 2011-05-08 04:24:47Z yegor256@yahoo.com $
 */
public class XmlRpcCommonsProxyAwareTransportFactory extends
    XmlRpcCommonsTransportFactory {

    /**
     * Proxy configuration.
     */
    private final ProxyConfig config = new ProxyConfig();

    /**
     * Creates a new instance.
     * @param pClient The client, which is controlling the factory.
     */
    public XmlRpcCommonsProxyAwareTransportFactory(final XmlRpcClient pClient) {
        super(pClient);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final void setHttpClient(final HttpClient pHttpClient) {
        if (pHttpClient != null) {
            if (StringUtils.isBlank(pHttpClient.getHostConfiguration()
                .getProxyHost())) {
                this.setProxyFromSystemProperties(pHttpClient);
            }
            super.setHttpClient(pHttpClient);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public final HttpClient getHttpClient() {
        HttpClient httpClient = super.getHttpClient();
        if (httpClient == null) {
            httpClient = new HttpClient();
        }
        this.setProxyFromSystemProperties(httpClient);
        return httpClient;
    }

    /**
     * Set the proxy in HttpClient if proxy data exists in system properties.
     * @param httpClient the HttpClient that will have proxy set
     */
    private void setProxyFromSystemProperties(final HttpClient httpClient) {
        if (this.config.getHost() != null) {
            httpClient.getHostConfiguration().setProxy(this.config.getHost(),
                this.config.getPort());
        }
    }

}
